#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

from enum import Enum

from psym.common.data_class import RecommendationSources, recommendations



class Entity(Enum):
    Location = "Location"
    LocationType = "LocationType"
    Equipment = "Equipment"
    EquipmentType = "EquipmentType"
    EquipmentPort = "EquipmentPort"
    EquipmentPortType = "EquipmentPortType"
    Link = "Link"
    Service = "Service"
    ServiceType = "ServiceType"
    ServiceEndpoint = "ServiceEndpoint"
    ServiceEndpointDefinition = "ServiceEndpointDefinition"
    SiteSurvey = "SiteSurvey"
    Customer = "Customer"
    Document = "Document"
    PropertyType = "PropertyType"
    Property = "Property"
    DocumentCategory = "DocumentCategory"
    User = "User"
    recommendations="recommendations"
    RecomendationSources="RecomendationSources"
    RecomendationsCategory="RecomendationsCategory"
    WorkOrder = "WorkOrder"
    WorkOrderType = "WorkOrderType"
    ProjectType = "ProjectType"
    Project = "Project"
    KqiCategory = "KqiCategory"
    KqiPerspective = "KqiPerspective"
    KqiTemporalFrecuency = "KqiTemporalFrecuency"
    KqiSource = "KqiSource"
    Kqi = "Kqi"
    KpiCategory = "KpiCategory"
    domain = "domain"
    tech = "tech"
    vendor = "vendor"
    NetworkType = "NetworkType"
    CounterFamiliy = "CounterFamiliy"
    Kpi = "Kpi"
    formula = "formula"
    counter = "counter"
    counterFormula = "counterFormula"
    ruleType= "ruleType"
    eventSeverity = "eventSeverity"
    comparator = "comparator"
    threshold = "threshold"
    rule= "rule"
    alarmStatus="alarmStatus"
    organization="organization"
    ruleLimit="ruleLimit"




#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from ...config import custom_scalars, datetime
from gql_client.runtime.variables import encode_variables
from gql import gql, Client
from gql.transport.exceptions import TransportQueryError
from functools import partial
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional
from time import perf_counter
from dataclasses_json import DataClassJsonMixin, config

from ..input.kqi_target_filter_input import KqiTargetFilterInput
from ..input.kqi_target_order import KqiTargetOrder


# fmt: off
QUERY: List[str] = ["""

query kqiTargets(
$after: Cursor
$first: Int
$before: Cursor
$last: Int
$orderBy: KqiTargetOrder
$filterBy: [KqiTargetFilterInput!]
){
  kqiTargets(after: $after, first: $first, before: $before, last: $last, orderBy: $orderBy, filterBy: $filterBy){
    totalCount
    edges{
      node{
        id
        name
        impact
        
        initTime
        endTime
        status
        kqi{
          id
          name
          description
          formula
          startDateTime
          endDateTime
        }
      }
      cursor
    }
    pageInfo{
      hasNextPage
      hasPreviousPage
      startCursor
      endCursor
    }
  }
}
"""
]


class kqiTargets:
    @dataclass(frozen=True)
    class kqiTargetsData(DataClassJsonMixin):
        @dataclass(frozen=True)
        class KqiTargetConnection(DataClassJsonMixin):
            @dataclass(frozen=True)
            class KqiTargetEdge(DataClassJsonMixin):
                @dataclass(frozen=True)
                class KqiTarget(DataClassJsonMixin):
                    @dataclass(frozen=True)
                    class Kqi(DataClassJsonMixin):
                        id: str
                        name: str
                        description: str
                        formula: str
                        startDateTime: datetime = _field(metadata=config(encoder=custom_scalars["Time"].encoder, decoder=custom_scalars["Time"].decoder, mm_field=custom_scalars["Time"].mm_field))
                        endDateTime: datetime = _field(metadata=config(encoder=custom_scalars["Time"].encoder, decoder=custom_scalars["Time"].decoder, mm_field=custom_scalars["Time"].mm_field))

                    id: str
                    name: str
                    impact: str
                    initTime: datetime = _field(metadata=config(encoder=custom_scalars["Time"].encoder, decoder=custom_scalars["Time"].decoder, mm_field=custom_scalars["Time"].mm_field))
                    endTime: datetime = _field(metadata=config(encoder=custom_scalars["Time"].encoder, decoder=custom_scalars["Time"].decoder, mm_field=custom_scalars["Time"].mm_field))
                    status: bool
                    kqi: Kqi

                node: Optional[KqiTarget]
                cursor: str

            @dataclass(frozen=True)
            class PageInfo(DataClassJsonMixin):
                hasNextPage: bool
                hasPreviousPage: bool
                startCursor: Optional[str]
                endCursor: Optional[str]

            totalCount: int
            edges: List[KqiTargetEdge]
            pageInfo: PageInfo

        kqiTargets: KqiTargetConnection

    # fmt: off
    @classmethod
    def execute(cls, client: Client, after: Optional[str] = None, first: Optional[int] = None, before: Optional[str] = None, last: Optional[int] = None, orderBy: Optional[KqiTargetOrder] = None, filterBy: List[KqiTargetFilterInput] = []) -> kqiTargetsData.KqiTargetConnection:
        variables: Dict[str, Any] = {"after": after, "first": first, "before": before, "last": last, "orderBy": orderBy, "filterBy": filterBy}
        new_variables = encode_variables(variables, custom_scalars)
        response_text = client.execute(
            gql("".join(set(QUERY))), variable_values=new_variables
        )
        res = cls.kqiTargetsData.from_dict(response_text)
        return res.kqiTargets

    # fmt: off
    @classmethod
    async def execute_async(cls, client: Client, after: Optional[str] = None, first: Optional[int] = None, before: Optional[str] = None, last: Optional[int] = None, orderBy: Optional[KqiTargetOrder] = None, filterBy: List[KqiTargetFilterInput] = []) -> kqiTargetsData.KqiTargetConnection:
        variables: Dict[str, Any] = {"after": after, "first": first, "before": before, "last": last, "orderBy": orderBy, "filterBy": filterBy}
        new_variables = encode_variables(variables, custom_scalars)
        response_text = await client.execute_async(
            gql("".join(set(QUERY))), variable_values=new_variables
        )
        res = cls.kqiTargetsData.from_dict(response_text)
        return res.kqiTargets



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from ...config import custom_scalars, datetime
from gql_client.runtime.variables import encode_variables
from gql import gql, Client
from gql.transport.exceptions import TransportQueryError
from functools import partial
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional
from time import perf_counter
from dataclasses_json import DataClassJsonMixin, config

from ..input.recommendations_filter_input import RecommendationsFilterInput
from ..input.recommendations_order import RecommendationsOrder


# fmt: off
QUERY: List[str] = ["""
query Recommendations(
$after: Cursor
$first: Int
$before: Cursor
$last: Int
$orderBy: RecommendationsOrder
$filterBy: [RecommendationsFilterInput!]
){
Recommendations(after: $after, first: $first, before: $before, last: $last, orderBy: $orderBy, filterBy: $filterBy){
totalCount
edges{
node{
    id
    externalID
    resource
    alarmType
    shortDescription
    longDescription
    command
    runbook
    priority
    status
    used
    vendor{id}
    recommendationsSources{id}
    recommendationsCategory{id}
    userCreate{
      name
      id}
    userApprobed{
      name
      id}

}
      cursor
    }
    pageInfo{
      hasNextPage
      hasPreviousPage
      startCursor
      endCursor
    }
  }
}

"""
]


class Recommendations:
    @dataclass(frozen=True)
    class RecommendationsData(DataClassJsonMixin):
        @dataclass(frozen=True)
        class RecommendationsConnection(DataClassJsonMixin):
            @dataclass(frozen=True)
            class RecommendationsEdge(DataClassJsonMixin):
                @dataclass(frozen=True)
                class Recommendations(DataClassJsonMixin):
                    @dataclass(frozen=True)
                    class Vendor(DataClassJsonMixin):
                        id: str

                    @dataclass(frozen=True)
                    class RecommendationsSources(DataClassJsonMixin):
                        id: str

                    @dataclass(frozen=True)
                    class RecommendationsCategory(DataClassJsonMixin):
                        id: str

                    @dataclass(frozen=True)
                    class User(DataClassJsonMixin):
                        name: str
                        id: str

                    id: str
                    externalID: str
                    resource: str
                    alarmType: str
                    shortDescription: str
                    longDescription: str
                    command: Optional[str]
                    runbook: Optional[str]
                    priority: int
                    status: bool
                    used: Optional[int]
                    vendor: Vendor
                    recommendationsSources: RecommendationsSources
                    recommendationsCategory: RecommendationsCategory
                    userCreate: User
                    userApprobed: Optional[User]

                node: Optional[Recommendations]
                cursor: str

            @dataclass(frozen=True)
            class PageInfo(DataClassJsonMixin):
                hasNextPage: bool
                hasPreviousPage: bool
                startCursor: Optional[str]
                endCursor: Optional[str]

            totalCount: int
            edges: List[RecommendationsEdge]
            pageInfo: PageInfo

        Recommendations: RecommendationsConnection

    # fmt: off
    @classmethod
    def execute(cls, client: Client, after: Optional[str] = None, first: Optional[int] = None, before: Optional[str] = None, last: Optional[int] = None, orderBy: Optional[RecommendationsOrder] = None, filterBy: List[RecommendationsFilterInput] = []) -> RecommendationsData.RecommendationsConnection:
        variables: Dict[str, Any] = {"after": after, "first": first, "before": before, "last": last, "orderBy": orderBy, "filterBy": filterBy}
        new_variables = encode_variables(variables, custom_scalars)
        response_text = client.execute(
            gql("".join(set(QUERY))), variable_values=new_variables
        )
        res = cls.RecommendationsData.from_dict(response_text)
        return res.Recommendations

    # fmt: off
    @classmethod
    async def execute_async(cls, client: Client, after: Optional[str] = None, first: Optional[int] = None, before: Optional[str] = None, last: Optional[int] = None, orderBy: Optional[RecommendationsOrder] = None, filterBy: List[RecommendationsFilterInput] = []) -> RecommendationsData.RecommendationsConnection:
        variables: Dict[str, Any] = {"after": after, "first": first, "before": before, "last": last, "orderBy": orderBy, "filterBy": filterBy}
        new_variables = encode_variables(variables, custom_scalars)
        response_text = await client.execute_async(
            gql("".join(set(QUERY))), variable_values=new_variables
        )
        res = cls.RecommendationsData.from_dict(response_text)
        return res.Recommendations



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from ...config import custom_scalars, datetime
from gql_client.runtime.variables import encode_variables
from gql import gql, Client
from gql.transport.exceptions import TransportQueryError
from functools import partial
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional
from time import perf_counter
from dataclasses_json import DataClassJsonMixin, config

from ..input.add_rule_input import AddRuleInput


# fmt: off
QUERY: List[str] = ["""
mutation addRule($input: AddRuleInput!) {
  addRule(input: $input) {
    id
    name
		gracePeriod
		ruleType {id}
		eventTypeName
		specificProblem
		additionalInfo
		status
		eventSeverity{id}
		threshold{id}

  }
}
"""
]


class addRule:
    @dataclass(frozen=True)
    class addRuleData(DataClassJsonMixin):
        @dataclass(frozen=True)
        class Rule(DataClassJsonMixin):
            @dataclass(frozen=True)
            class RuleType(DataClassJsonMixin):
                id: str

            @dataclass(frozen=True)
            class EventSeverity(DataClassJsonMixin):
                id: str

            @dataclass(frozen=True)
            class Threshold(DataClassJsonMixin):
                id: str

            id: str
            name: str
            gracePeriod: Optional[int]
            ruleType: RuleType
            eventTypeName: Optional[str]
            specificProblem: Optional[str]
            additionalInfo: Optional[str]
            status: bool
            eventSeverity: EventSeverity
            threshold: Threshold

        addRule: Rule

    # fmt: off
    @classmethod
    def execute(cls, client: Client, input: AddRuleInput) -> addRuleData.Rule:
        variables: Dict[str, Any] = {"input": input}
        new_variables = encode_variables(variables, custom_scalars)
        response_text = client.execute(
            gql("".join(set(QUERY))), variable_values=new_variables
        )
        res = cls.addRuleData.from_dict(response_text)
        return res.addRule

    # fmt: off
    @classmethod
    async def execute_async(cls, client: Client, input: AddRuleInput) -> addRuleData.Rule:
        variables: Dict[str, Any] = {"input": input}
        new_variables = encode_variables(variables, custom_scalars)
        response_text = await client.execute_async(
            gql("".join(set(QUERY))), variable_values=new_variables
        )
        res = cls.addRuleData.from_dict(response_text)
        return res.addRule



#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.


from psym.client import SymphonyClient
from psym.common.data_class import counterFormula, formula, counter

from ..graphql.input.edit_counter_formula_input import EditCounterFormulaInput
from ..graphql.input.add_counter_formula_input import AddCounterFormulaInput
from ..graphql.mutation.add_counter_formula import addCounterFormula
from ..graphql.mutation.edit_counter_formula import editCounterFormula
from ..graphql.mutation.remove_counter_formula import removeCounterFormula
from psym.common.constant import PAGINATION_STEP
from typing import Any, Dict, Iterator, List, Optional




def add_counter_formula(
    client: SymphonyClient,  mandatory: bool, counter: str,  formula: str
) -> counterFormula:
    """This function adds Counter Formula.
    
    :param mandatory: mandatory
    :type mandatory: str
    :param id: ID
    :type id: str
    :param counter: counter
    :type counter: `~psym.common.data_class.counter`
    :param formula: formula
    :type formula: `~psym.common.data_class.formula`
    

    :return: CounterFormula object
    :rtype: :class:`~psym.common.data_class.counterFormula`

   
    **Example 1**

    .. code-block:: python

        counter_family = client.add_counter_family(
            name="counter_family",
        )

        new_vendor = client.add_vendor(
            name="vendor",
        )

        counter = client.add_counter(
            name="counter",
            externalID="new counter",
            networkManagerSystem="counter",
            counterFamily=counter_family.id,
            vendor= vendor.id
        )

        domain = client.add_domain(
            name="domain",
        )
        kpi_category = client.add_kpi_category(
            name="kpi_category",
        )

        Kpi = client.add_Kpi(
            name="Kpi_2",
            description="new kpi",
            status=True,
            domain=domain.id,
            kpiCategory=kpi_category.id
        )

        domain = client.add_domain(
            name="domain",
        )
        
        new_Tech = client.add_Tech(
            name="new_Tech", 
            domain=domain.id
            )

        network_type = client.add_network_type(
            name="network_type",
        )

        new_formula = client.add_formula(
            textFormula="formula_1",
            status=True,
            kpi=Kpi_created.id,
            tech=tech_created.id,
            networkType=network_type.id
        )

        new_counter_formula = client.add_counter_formula(
            name="counter_formula",
            mandatory=True,
            counter=counter.id,
            formula=formula.id
        )
        print(new_counter_formula)
    """
    domain_input = AddCounterFormulaInput(
    mandatory=mandatory,
    counterFk=counter,
    formulaFk=formula)
    result = addCounterFormula.execute(client, input=domain_input)
    return counterFormula(mandatory=result.mandatory,
    id=result.id,  
    counter=result.counterFk, 
    formula=result.formulaFk)

def edit_counter_formula(
    client: SymphonyClient,
    CounterFormula: counterFormula,
    new_mandatory: Optional[bool] = None,
    formula:formula= None,
    counter: counter = None,
) -> None:
    """This function edits counterFormula.

    :param mandatory: mandatory
    :type mandatory: str
    :param id: ID
    :type id: str
    :param counter: counter
    :type counter: psym.common.data_class.counter
    :param formula: formula
    :type formula: psym.common.data_class.formula

    :return: none object
    :rtype: :class:`~psym.common.data_class.counterFormula`

    **Example 1**

    .. code-block:: python

        counter_formula_edited = client.edit_counter_formula(
            CounterFormula=CounterFormula,
            new_mandatory=new_mandatory,
            counter=counter.id,
            formula=formula.id
            )
    """
    params: Dict[str, Any] = {}
    if new_mandatory is not None:
        params.update({"mandatory": new_mandatory})
    if new_mandatory is not None:
        editCounterFormula.execute(client, input=EditCounterFormulaInput(
            id=CounterFormula.id, 
            mandatory=new_mandatory,
            counterFk=counter,
            formulaFk=formula))

def delete_counter_formula(client: SymphonyClient, id: str) -> None:
    """This function deletes CounterFormula.

    :param id: CounterFormula ID
    :type id: str

    :raises:
        FailedOperationException: Internal symphony error

    **Example**

    .. code-block:: python

        client.delete_counter_formula(id="12345678")
    """
    removeCounterFormula.execute(client, id=id)



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from functools import partial
from ...config import custom_scalars, datetime
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional

from dataclasses_json import DataClassJsonMixin, config

from gql_client.runtime.enum_utils import enum_field_metadata
from ..enum.image_entity import ImageEntity


@dataclass(frozen=True)
class AddImageInput(DataClassJsonMixin):
    entityType: ImageEntity = _field(metadata=enum_field_metadata(ImageEntity))
    entityId: str
    imgKey: str
    fileName: str
    fileSize: int
    modified: datetime = _field(metadata=config(encoder=custom_scalars["Time"].encoder, decoder=custom_scalars["Time"].decoder, mm_field=custom_scalars["Time"].mm_field))
    contentType: str
    category: Optional[str] = None
    annotation: Optional[str] = None
    documentCategoryId: Optional[str] = None



#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.




from psym.client import SymphonyClient
from psym.common.data_class import KqiCategory

from ..graphql.input.edit_kqi_category_input import EditKqiCategoryInput
from ..graphql.input.add_kqi_category_input import AddKqiCategoryInput
from ..graphql.mutation.add_kqi_category import addKqiCategory
from ..graphql.mutation.edit_kqi_category import EditKqiCategory
from ..graphql.input.edit_kqi_category_input import EditKqiCategoryInput
from ..graphql.query.kqi_category import kqiCategories
from ..graphql.mutation.remove_kqi_category import removeKqiCategory
from psym.common.constant import PAGINATION_STEP
from typing import Any, Dict, Iterator, List, Optional



def add_kqi_category(
    client: SymphonyClient, name: str
) -> KqiCategory:
    """This function adds Kqi Category.

    :param name: Kqi Category name
    :type name: str

    :return: kqiCategory object
    :rtype: :class:`~psym.common.data_class.kqiCategory`

    **Example 1**

    .. code-block:: python

        new_kqi_category = client.add_kqi_category(
            name="kqi_category",

        )
    """
    kqi_category_input = AddKqiCategoryInput(name=name)
    if (len(name) != 0):
        result = addKqiCategory.execute(client, input=kqi_category_input)
        return KqiCategory(name=result.name, id=result.id)
    else:
        print ("you cant leave the name parameter empty")


def edit_kqi_category(
    client: SymphonyClient,
    kpicategory: KqiCategory,
    new_name: Optional[str] = None,
) -> None:
    """This function edits Kqi Category.

    :param kqi_category: Kqi Category entity
    :type name: str
    :param new_name: Kqi Category name
    :type name: str

    :return: none object
    :rtype: :class:`~psym.common.data_class.kqiCategory`

    **Example 1**

    .. code-block:: python

        new_kqi_category_edited = client.edit_kqi_category(
            kqi_category=kqiCategory,
            new_name="kqi_category_edited",

        )
        print(new_kqi_category_edited)
    """
    params: Dict[str, Any] = {}
    if new_name is not None:
        params.update({"_name_": new_name})
    if new_name is not None:
        EditKqiCategory.execute(client, input=EditKqiCategoryInput(id=kpicategory.id, name=new_name))


def get_kqi_categories(client: SymphonyClient) -> Iterator[KqiCategory]:
    """ this funtion Get Kqi Categories


    :return: kqiCategory object
    :rtype: Iterator[ :class:`~psym.common.data_class.kqiCategory` ]

    **Example**

    .. code-block:: python

        kqi_categoryes = client.get_kqi_categories()
        for kqi_category in kqi_categories:
            print(kqi_category.name)
    """
    kqi_categories = kqiCategories.execute(client, first=PAGINATION_STEP)
    edges = kqi_categories.edges if kqi_categories else []
    while kqi_categories is not None and kqi_categories.pageInfo.hasNextPage:
        kqi_categories = kqiCategories.execute(
            client, after=kqi_categories.pageInfo.endCursor, first=PAGINATION_STEP
        )
        if kqi_categories is not None:
            edges.extend(kqi_categories.edges)

    for edge in edges:
        node = edge.node
        if node is not None:
            yield KqiCategory(
                id=node.id,
                name=node.name,
            )


def delete_kqi_category(client: SymphonyClient, id: str) -> None:
    """This function delete Kqi Category.

    :param name: Kqi Category name
    :type name: :class:`~psym.common.data_class.kqiCategory`
    :rtype: None

    **Example**

    .. code-block:: python

        client.delete_kqi_category(id=123456789)
    """
	
    removeKqiCategory.execute(client, id=id)






#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from enum import Enum


class VariableType(Enum):
    STRING = "STRING"
    INT = "INT"
    DATE = "DATE"
    WORK_ORDER = "WORK_ORDER"
    WORK_ORDER_TYPE = "WORK_ORDER_TYPE"
    LOCATION = "LOCATION"
    PROJECT = "PROJECT"
    USER = "USER"
    MISSING_ENUM = ""

    @classmethod
    def _missing_(cls, value: object) -> "VariableType":
        return cls.MISSING_ENUM



#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.


from psym.common.data_class import RecommendationSources
from psym.client import SymphonyClient

from ..graphql.input.edit_recommendations_sources_input import EditRecommendationsSourcesInput
from ..graphql.input.add_recommendations_sources_input import AddRecommendationsSourcesInput
from ..graphql.mutation.add_recommendations_sources import addRecommendationsSources
from ..graphql.mutation.edit_recommendations_sources import editRecommendationsSources
from ..graphql.mutation.remove_recommendations_sources import removeRecommendationSources
from ..graphql.query.recommendation_sources import RecommendationsSources
from psym.common.constant import PAGINATION_STEP
from typing import Any, Dict, Iterator, List, Optional




def add_recommendations_sources(
    client: SymphonyClient, name: str
) -> RecommendationSources:
    """This function adds recommendations Sources.

    :param name: recommendations Sources name
    :type name: str

    :return: recommendationsSources object
    :rtype: :class:`~psym.common.data_class.recommendationsSources`

    **Example 1**

    .. code-block:: python

        new_recommendations_sources = client.add_recommendations_sources(
            name="recommendations_sources",
        )
        print(new_recommendations_sources)
    """
    recommendations_sources_input = AddRecommendationsSourcesInput(name=name)
    if (len(name) != 0):
        result = addRecommendationsSources.execute(client, input=recommendations_sources_input)
        return RecommendationSources(name=result.name, id=result.id)
    else:
        print ("you cant leave the name parameter empty")

def edit_recommendations_sources(
    client: SymphonyClient,
    recommendation_sources: RecommendationSources,
    new_name: Optional[str] = None,
) -> None:
    """This function edits recommendations Sources.

    :param recommendations_sources: recommendations Sources entity
    :type name: str
    :param new_name: recommendations Sources name
    :type name: str

    :return: none object
    :rtype: :class:`~psym.common.data_class.recommendationsSources`

    **Example 1**

    .. code-block:: python

        new_recommendations_sources_edited = client.edit_recommendations_sources(
            recommendations_sources=recommendationsSources,
            new_name="recommendations_sources_edited",
        )

    """
    params: Dict[str, Any] = {}
    if new_name is not None:
        params.update({"_name_": new_name})
    if new_name is not None:
        editRecommendationsSources.execute(client, input=EditRecommendationsSourcesInput(id=recommendation_sources.id, name=new_name))


def get_recommendations_sources(client: SymphonyClient) -> Iterator[RecommendationsSources]:
    """ this funtion Get recommendations Sourceses


    :return: recommendationsSources object
    :rtype: Iterator[ :class:`~psym.common.data_class.recommendationsSources` ]

    **Example**

    .. code-block:: python

        recommendations_sources = client.get_recommendations_sources()
        for recommendations_sources in recommendations_sources:
            print(recommendations_sources.name)
    """
    recommendations_sourcesess_ = RecommendationsSources.execute(client, first=PAGINATION_STEP)
    edges = recommendations_sourcesess_.edges if recommendations_sourcesess_ else []
    while recommendations_sourcesess_ is not None and recommendations_sourcesess_.pageInfo.hasNextPage:
        recommendations_sourcesess_ = RecommendationsSources.execute(
            client, after=recommendations_sourcesess_.pageInfo.endCursor, first=PAGINATION_STEP
        )
        if recommendations_sourcesess_ is not None:
            edges.extend(recommendations_sourcesess_.edges)

    for edge in edges:
        node = edge.node
        if node is not None:
            yield RecommendationSources(
                id=node.id,
                name=node.name,
            )


def remove_recommendations_sources(client: SymphonyClient, id: str) -> None:
    """This function delete recommendations Sources.

    :param name: recommendations Sources name
    :type name: :class:`~psym.common.data_class.recommendationsSources`
    :rtype: None

    **Example**

    .. code-block:: python

        client.remove_recommendations_sources(id=123456789)
    """
    removeRecommendationSources.execute(client, id=id)

    


#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

from numbers import Number
from typing import Dict, Iterator, List, Optional, Sequence, Tuple, cast

from psym.client import SymphonyClient
from psym.common.cache import LOCATION_TYPES
from psym.common.constant import LOCATIONS_TO_SEARCH, PAGINATION_STEP
from psym.common.data_class import Document, ImageEntity, Location, PropertyValue
from psym.common.data_enum import Entity
from psym.common.data_format import format_to_location

from .._utils import get_graphql_property_inputs, update_property_input_ids
from ..exceptions import (
    EntityNotFoundError,
    LocationCannotBeDeletedWithDependency,
    LocationIsNotUniqueException,
)
from ..graphql.enum.filter_operator import FilterOperator
from ..graphql.fragment.location import LocationFragment
from ..graphql.input.add_location_input import AddLocationInput
from ..graphql.input.edit_location_input import EditLocationInput
from ..graphql.input.location_filter_input import (
    LocationFilterInput,
    LocationFilterType,
)
from ..graphql.mutation.add_location import AddLocationMutation
from ..graphql.mutation.edit_location import EditLocationMutation
from ..graphql.mutation.move_location import MoveLocationMutation
from ..graphql.mutation.remove_location import RemoveLocationMutation
from ..graphql.query.get_locations import GetLocationsQuery
from ..graphql.query.location_children import LocationChildrenQuery
from ..graphql.query.location_deps import LocationDepsQuery
from ..graphql.query.location_documents import LocationDocumentsQuery
from ..graphql.query.location_search import LocationSearchQuery


def _get_locations_by_name_and_type(
    client: SymphonyClient, location_name: str, location_type_name: str
) -> Sequence[LocationFragment]:
    location_filters = [
        LocationFilterInput(
            filterType=LocationFilterType.LOCATION_TYPE,
            operator=FilterOperator.IS_ONE_OF,
            idSet=[LOCATION_TYPES[location_type_name].id],
            stringSet=[],
        ),
        LocationFilterInput(
            filterType=LocationFilterType.LOCATION_INST_NAME,
            operator=FilterOperator.IS,
            stringValue=location_name,
            idSet=[],
            stringSet=[],
        ),
    ]

    result = LocationSearchQuery.execute(
        client, filters=location_filters, limit=LOCATIONS_TO_SEARCH
    )
    locations = []
    if result is not None:
        for edge in result.edges:
            node = edge.node
            if node is not None:
                locations.append(node)
    return locations


def _get_location_children_by_name_and_type(
    client: SymphonyClient,
    location_name: str,
    location_type_name: str,
    current_location: Optional[LocationFragment],
) -> Sequence[LocationFragment]:
    if current_location is None:
        return _get_locations_by_name_and_type(
            client=client,
            location_name=location_name,
            location_type_name=location_type_name,
        )

    else:
        location_id = current_location.id
        location_with_children = LocationChildrenQuery.execute(client, id=location_id)
        if location_with_children is None:
            raise EntityNotFoundError(entity=Entity.Location, entity_id=location_id)

        return [
            location
            for location in location_with_children.children
            if location.locationType.name == location_type_name
            and location.name == location_name
        ]


def add_location(
    client: SymphonyClient,
    location_hirerchy: List[Tuple[str, str]],
    properties_dict: Dict[str, PropertyValue],
    lat: Optional[float] = None,
    long: Optional[float] = None,
    external_id: Optional[str] = None,
) -> Location:
    """Create a new location of a specific type with a specific name.
    It will also get the requested location specifiers for hirerchy
    leading to it and will create all the hirerchy.
    However the `lat`,`long` and `properties_dict` would only apply for the last location in the chain.
    If a location with its name in this place already exists, then existing location is returned

    :param location_hirerchy: Locations hierarchy
    :type location_hirerchy: List[Tuple[str, str]]

        * str - location type name
        * str - location name

    :param properties_dict: Dictionary of property name to property value

        * str - property name
        * PropertyValue - new value of the same type for this property

    :type properties_dict: Dict[str, PropertyValue]
    :param lat: Latitude
    :type lat: float, optional
    :param long: Longitude
    :type long: float, optional
    :param external_id: Location external ID
    :type external_id: str, optional

    :raises:
        * LocationIsNotUniqueException: There is more than one location to return
          in the chain and it is not clear where to create or what to return
        * FailedOperationException: Internal symphony error
        * :class:`~psym.exceptions.EntityNotFoundError`: Parent location in the chain does not exist

    :return: Location object
    :rtype: :class:`~psym.common.data_class.Location`

    **Example**

    .. code-block:: python

        location = client.add_location(
            location_hirerchy=[
                ("Country", "England"),
                ("City", "Milton Keynes"),
                ("Site", "Bletchley Park")
            ],
            properties_dict={
                "Date Property": date.today(),
                "Lat/Lng Property": (-1.23,9.232),
                "E-mail Property": "user@fb.com",
                "Number Property": 11,
                "String Property": "aa",
                "Float Property": 1.23
            },
            lat=-11.32,
            long=98.32,
            external_id=None,
        )
    """

    last_location: Optional[LocationFragment] = None

    for i, location in enumerate(location_hirerchy):

        location_type = location[0]
        location_name = location[1]
        properties = []
        lat_val = None
        long_val = None

        if i == len(location_hirerchy) - 1:
            property_types = LOCATION_TYPES[location_type].property_types
            properties = get_graphql_property_inputs(property_types, properties_dict)
            lat_val = cast(Number, lat)
            long_val = cast(Number, long)

        location_search_result = _get_location_children_by_name_and_type(
            client=client,
            location_name=location_name,
            location_type_name=location_type,
            current_location=last_location,
        )
        if not location_search_result:
            last_location = AddLocationMutation.execute(
                client=client,
                input=AddLocationInput(
                    name=location_name,
                    type=LOCATION_TYPES[location_type].id,
                    latitude=lat_val,
                    longitude=long_val,
                    parent=last_location.id if last_location else None,
                    properties=properties,
                    externalID=external_id,
                ),
            )

        elif len(location_search_result) == 1:
            last_location = location_search_result[0]

        elif len(location_search_result) > 1:
            raise LocationIsNotUniqueException(
                location_name=location_name, location_type=location_type
            )
    if last_location is None:
        raise EntityNotFoundError(
            entity=Entity.Location, msg=f"<location_hierarchy: {location_hirerchy}>"
        )

    return format_to_location(location_fragment=last_location)


def get_location(
    client: SymphonyClient, location_hirerchy: List[Tuple[str, str]]
) -> Location:
    """This function returns a location of a specific type with a specific name.
    It gets the requested location specifiers or the hirerchy leading to it

    :param location_hirerchy: Locations hierarchy
    :type location_hirerchy: List[Tuple[str, str]]

        * str - location type name
        * str - location name

    :raises:
        * LocationIsNotUniqueException: There is more than one location to return
          in the chain and it is not clear where to create or what to return
        * LocationNotFoundException: Location was not found
        * FailedOperationException: Internal symphony error
        * :class:`~psym.exceptions.EntityNotFoundError`: Location in the chain does not exist

    :return: Location object
    :rtype: :class:`~psym.common.data_class.Location`

    **Example**

    .. code-block:: python

        location = client.get_location(
            location_hirerchy=[
                ("Country", "England"),
                ("City", "Milton Keynes"),
                ("Site", "Bletchley Park")
            ])

    .. code-block:: python

        # this call will fail if there is Bletchley Park in two cities
        location = client.get_location(location_hirerchy=[("Site", "Bletchley Park")])
    """

    last_location = None

    for location in location_hirerchy:
        location_type = location[0]
        location_name = location[1]

        location_search_result = _get_location_children_by_name_and_type(
            client=client,
            location_name=location_name,
            location_type_name=location_type,
            current_location=last_location,
        )

        if not location_search_result:
            raise EntityNotFoundError(entity=Entity.Location, entity_name=location_name)

        if len(location_search_result) > 1:
            raise LocationIsNotUniqueException(
                location_name=location_name, location_type=location_type
            )

        last_location = location_search_result[0]

    if last_location is None:
        raise EntityNotFoundError(
            entity=Entity.Location, msg=f"<location_hierarchy: {location_hirerchy}>"
        )
    return format_to_location(location_fragment=last_location)


def get_locations(client: SymphonyClient) -> Iterator[Location]:
    """This function returns all existing locations

    :return: Locations Iterator
    :rtype: Iterator[ :class:`~psym.common.data_class.Location` ]

    **Example**

    .. code-block:: python

        all_locations = client.get_locations()
    """

    def generate_pages(
        client: SymphonyClient,
    ) -> Iterator[GetLocationsQuery.GetLocationsQueryData.LocationConnection]:
        locations = GetLocationsQuery.execute(client, first=PAGINATION_STEP)
        if locations:
            yield locations
        while locations is not None and locations.pageInfo.hasNextPage:
            locations = GetLocationsQuery.execute(
                client, after=locations.pageInfo.endCursor, first=PAGINATION_STEP
            )
            if locations is not None:
                yield locations

    for page in generate_pages(client):
        for edge in page.edges:
            node = edge.node
            if node is not None:
                yield format_to_location(location_fragment=node)


def get_location_children(
    client: SymphonyClient, location_id: str
) -> Iterator[Location]:
    """This function returns all children locations of the given location

    :param location_id: Parent location ID
    :type location_id: str

    :raises:
        :class:`~psym.exceptions.EntityNotFoundError`: Location does not exist

    :return: Locations Iterator
    :rtype: Iterator[ :class:`~psym.common.data_class.Location` ]

    **Example**

    .. code-block:: python

        client.add_location(
            [
                ("Country", "England"),
                ("City", "Milton Keynes"),
            ],
            {},
        )
        client.add_location(
            [
                ("Country", "England"),
                ("City", "London"),
            ],
            {},
        )
        parent_location = client.get_location(
            location_hirerchy=[
                ("Country", "England"),
            ],
        )
        children_locations = client.get_location_children(location_id=parent_location.id)
        # This call will return a list with 2 locations: "Milton Keynes" and "London"
    """
    location_with_children = LocationChildrenQuery.execute(client, id=location_id)
    if not location_with_children:
        raise EntityNotFoundError(entity=Entity.Location, entity_id=location_id)

    for location in location_with_children.children:
        yield format_to_location(location_fragment=location)


def edit_location(
    client: SymphonyClient,
    location: Location,
    new_name: Optional[str] = None,
    new_lat: Optional[float] = None,
    new_long: Optional[float] = None,
    new_external_id: Optional[str] = None,
    new_properties: Optional[Dict[str, PropertyValue]] = None,
) -> Location:
    """This function returns edited location.

    :param location: Location object
    :type location: :class:`~psym.common.data_class.Location`
    :param new_name: Location new name
    :type new_name: str, optional
    :param new_lat: Location new latitude
    :type new_lat: float, optional
    :param new_long: Location new longitude
    :type new_long: float, optional
    :param new_external_id: Location new external ID
    :type new_external_id: str, optional
    :param new_properties: Dictionary of property name to property value

        * str - property name
        * PropertyValue - new value of the same type for this property

    :type new_properties: Dict[str, PropertyValue], optional

    :raises:
        FailedOperationException: Internal symphony error

    :return: Location object
    :rtype: :class:`~psym.common.data_class.Location`

    **Example**

    .. code-block:: python

        # this call will fail incase the 'Bletchley Park' in two cities
        location = client.get_location(location_hirerchy=[("Site", "Bletchley Park")])
        edited_location = client.edit_location(
            location=location,
            new_name="New Bletchley Park",
            new_lat=10,
            new_long=20,
            new_external_id=None,
            new_properties={"Contact": "new_contact@info.com"},
        )
    """
    properties = []
    location_type = location.location_type_name
    if new_properties:
        property_types = LOCATION_TYPES[location_type].property_types
        existing_properties = {p.propertyType.id: p for p in location.properties}
        property_inputs = get_graphql_property_inputs(property_types, new_properties)
        properties = update_property_input_ids(existing_properties, property_inputs)

    if new_external_id is None:
        new_external_id = location.external_id
    edit_location_input = EditLocationInput(
        id=location.id,
        name=new_name if new_name is not None else location.name,
        latitude=cast(Number, new_lat) if new_lat is not None else location.latitude,
        longitude=cast(Number, new_long)
        if new_long is not None
        else location.longitude,
        properties=properties,
        externalID=new_external_id,
    )
    result = EditLocationMutation.execute(client, edit_location_input)
    return format_to_location(location_fragment=result)


def delete_location(client: SymphonyClient, location: Location) -> None:
    """This delete existing location.

    :param location: Location object
    :type location: :class:`~psym.common.data_class.Location`

    :raises:
        * LocationCannotBeDeletedWithDependency: Location has dependencies in one or more
          ["files", "images", "children", "surveys", "equipment"]
        * FailedOperationException: Internal symphony error
        * :class:`~psym.exceptions.EntityNotFoundError`: Location does not exist

    :rtype: None

    **Example**

    .. code-block:: python

        location = client.get_location(location_hirerchy=[('Site', 'Bletchley Park')])
        client.delete_location(location=location)
    """
    location_with_deps = LocationDepsQuery.execute(client, id=location.id)
    if location_with_deps is None:
        raise EntityNotFoundError(entity=Entity.Location, entity_id=location.id)
    if len(location_with_deps.files) > 0:
        raise LocationCannotBeDeletedWithDependency(location.name, "files")
    if len(location_with_deps.images) > 0:
        raise LocationCannotBeDeletedWithDependency(location.name, "images")
    if len(location_with_deps.children) > 0:
        raise LocationCannotBeDeletedWithDependency(location.name, "children")
    if len(location_with_deps.surveys) > 0:
        raise LocationCannotBeDeletedWithDependency(location.name, "surveys")
    if len(location_with_deps.equipments) > 0:
        raise LocationCannotBeDeletedWithDependency(location.name, "equipment")
    RemoveLocationMutation.execute(client, id=location.id)


def move_location(
    client: SymphonyClient, location_id: str, new_parent_id: Optional[str]
) -> Location:
    """This function moves existing location to another existing parent location.

    :param location_id: Existing location ID to be moved
    :type location_id: str
    :param new_parent_id: New existing parent location ID
    :type new_parent_id: str, optional

    :raises:
        FailedOperationException: Internal symphony error

    :return: Location object
    :rtype: :class:`~psym.common.data_class.Location`

    **Example**

    .. code-block:: python

        # this call will fail if there is Bletchley Park in two cities
        location = client.get_location(location_hirerchy=[("Site", "Bletchley Park")])
        moved_location = client.move_locatoin(
            location_id=location.id,
            new_parent_id="12345"
        )
    """
    result = MoveLocationMutation.execute(
        client, locationID=location_id, parentLocationID=new_parent_id
    )
    return format_to_location(location_fragment=result)


def get_location_by_external_id(client: SymphonyClient, external_id: str) -> Location:
    """This function returns location by external ID.

    :param external_id: Location external ID
    :type external_id: str

    :raises:
        * LocationNotFoundException: Location with this external ID does not exists
        * FailedOperationException: Internal symphony error
        * :class:`~psym.exceptions.EntityNotFoundError`: Location does not exist

    :return: Location object
    :rtype: :class:`~psym.common.data_class.Location`

    **Example**

    .. code-block:: python

        location = client.get_location_by_external_id(external_id="12345")
    """
    location_filter = LocationFilterInput(
        filterType=LocationFilterType.LOCATION_INST_EXTERNAL_ID,
        operator=FilterOperator.IS,
        stringValue=external_id,
        idSet=[],
        stringSet=[],
    )

    location_search_result = LocationSearchQuery.execute(
        client, filters=[location_filter], limit=LOCATIONS_TO_SEARCH
    )

    if not location_search_result or location_search_result.totalCount == 0:
        raise EntityNotFoundError(
            entity=Entity.Location, msg=f"<external_id: {external_id}"
        )
    if location_search_result.totalCount > 1:
        raise LocationIsNotUniqueException(external_id=external_id)

    for edge in location_search_result.edges:
        node = edge.node
        if node is not None:
            return format_to_location(location_fragment=node)
    raise EntityNotFoundError(
        entity=Entity.Location, msg=f"<external_id: {external_id}"
    )


def get_location_documents(
    client: SymphonyClient, location: Location
) -> List[Document]:
    """This function returns locations documents.

    :param location: Location object
    :type location: :class:`~psym.common.data_class.Location`

    :raises:
        * FailedOperationException: Internal symphony error
        * :class:`~psym.exceptions.EntityNotFoundError`: Location does not exist

    :return: Documents List
    :rtype: List[ :class:`~psym.common.data_class.Document` ]

    **Example**

    .. code-block:: python

        # this call will fail if there is Bletchley Park in two cities
        location = client.get_location(location_hirerchy=[("Site", "Bletchley Park")])
        location = client.get_location_documents(location=location)
    """
    location_with_documents = LocationDocumentsQuery.execute(client, id=location.id)
    if not location_with_documents:
        raise EntityNotFoundError(entity=Entity.Location, entity_id=location.id)
    files = [
        Document(
            name=file.fileName,
            id=file.id,
            parent_id=location.id,
            parent_entity=ImageEntity.LOCATION,
            category=file.category,
            document_category=file.documentCategory
        )
        for file in location_with_documents.files
    ]
    images = [
        Document(
            name=file.fileName,
            id=file.id,
            parent_id=location.id,
            parent_entity=ImageEntity.LOCATION,
            category=file.category,
            document_category=file.documentCategory
        )
        for file in location_with_documents.images
    ]

    return files + images



#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

from typing import Dict, Iterator

from psym.client import SymphonyClient
from psym.common.cache import PORT_TYPES
from psym.common.constant import PAGINATION_STEP
from psym.common.data_class import (
    Equipment,
    EquipmentPort,
    EquipmentPortDefinition,
    EquipmentPortDefinitionAlias,
    Link,
    PropertyValue,
)
from psym.common.data_enum import Entity

from .._utils import get_graphql_property_inputs
from ..exceptions import EntityNotFoundError, EquipmentPortIsNotUniqueException
from ..graphql.input.edit_equipment_port_input import EditEquipmentPortInput
from ..graphql.input.link_side import LinkSide
from ..graphql.mutation.edit_equipment_port import EditEquipmentPortMutation
from ..graphql.mutation.edit_link import EditLinkInput, EditLinkMutation
from ..graphql.query.equipment_ports import EquipmentPortsQuery
from ..graphql.query.ports import PortsQuery


def get_port(
    client: SymphonyClient, equipment: Equipment, port_name: str
) -> EquipmentPort:
    """This function returns port in equipment based on its name.

    :param equipment: Existing equipment object
    :type equipment: :class:`~psym.common.data_class.Equipment`
    :param port_name: Existing port name
    :type port_name: str

    :raises:
        * EquipmentPortIsNotUniqueException: There is more than one port with this name
        * :class:`~psym.exceptions.EntityNotFoundError`: Equipment does not exist or port was not found

    :return: EquipmentPort object
    :rtype: :class:`~psym.common.data_class.EquipmentPort`

    **Example**

    .. code-block:: python

        location = client.get_location(location_hirerchy=[("Country", "LS_IND_Prod_Copy")])
        equipment = client.get_equipment(name="indProdCpy1_AIO", location=location)
        port = client.get_port(equipment=equipment, port_name="Z AIO - Port 1")
    """
    equipment_with_ports = EquipmentPortsQuery.execute(client, id=equipment.id)

    if not equipment_with_ports:
        raise EntityNotFoundError(entity=Entity.Equipment, entity_id=equipment.id)

    ports = [
        port for port in equipment_with_ports.ports if port.definition.name == port_name
    ]

    if len(ports) > 1:
        raise EquipmentPortIsNotUniqueException(equipment.name, port_name)
    if len(ports) == 0:
        raise EntityNotFoundError(entity=Entity.EquipmentPort, entity_name=port_name)

    port_type_name = None
    port_type = ports[0].definition.portType
    if port_type is not None:
        port_type_name = port_type.name
    link = ports[0].link

    return EquipmentPort(
        id=ports[0].id,
        properties=ports[0].properties,
        definition=EquipmentPortDefinition(
            id=ports[0].definition.id,
            name=ports[0].definition.name,
            port_type_name=port_type_name,
            visible_label=ports[0].definition.visibleLabel,
            port_definition_index=ports[0].definition.index,
            connected_ports=[
                EquipmentPortDefinitionAlias(id=cp.id, name=cp.name)
                for cp in ports[0].definition.connectedPorts
            ],
        ),
        link=Link(
            id=link.id,
            properties=link.properties,
            service_ids=[s.id for s in link.services],
        )
        if link
        else None,
    )


def get_ports(client: SymphonyClient) -> Iterator[EquipmentPort]:
    """This function returns all existing ports

    :return: EquipmentPorts Iterator
    :rtype: Iterator[ :class:`~psym.common.data_class.EquipmentPort` ]

    **Example**

    .. code-block:: python

        all_ports = client.get_ports()
    """

    def generate_pages(
        client: SymphonyClient,
    ) -> Iterator[PortsQuery.PortsQueryData.EquipmentPortConnection]:
        ports = PortsQuery.execute(client, first=PAGINATION_STEP)
        if ports:
            yield ports
        while ports is not None and ports.pageInfo.hasNextPage:
            ports = PortsQuery.execute(
                client, after=ports.pageInfo.endCursor, first=PAGINATION_STEP
            )
            if ports is not None:
                yield ports

    for page in generate_pages(client):
        for edge in page.edges:
            node = edge.node
            if node is not None:
                port_type = None
                if node.definition.portType is not None:
                    port_type = node.definition.portType
                link = None
                if node.link is not None:
                    link = node.link
                yield EquipmentPort(
                    id=node.id,
                    properties=node.properties,
                    definition=EquipmentPortDefinition(
                        id=node.definition.id,
                        name=node.definition.name,
                        port_type_name=port_type.name if port_type else None,
                        visible_label=node.definition.visibleLabel,
                        port_definition_index=node.definition.index,
                        connected_ports=[
                            EquipmentPortDefinitionAlias(id=cp.id, name=cp.name)
                            for cp in node.definition.connectedPorts
                        ],
                    ),
                    link=Link(
                        id=link.id,
                        properties=link.properties,
                        service_ids=[s.id for s in link.services],
                    )
                    if link
                    else None,
                )


def edit_port_properties(
    client: SymphonyClient,
    equipment: Equipment,
    port_name: str,
    new_properties: Dict[str, PropertyValue],
) -> EquipmentPort:
    """This function returns edited port in equipment based on its name.

    :param equipment: Existing equipment object
    :type equipment: :class:`~psym.common.data_class.Equipment`
    :param port_name: Equipment type name
    :type port_name: str
    :param new_properties: Dictionary of property name to property value

        * str - property name
        * PropertyValue - new value of the same type for this property

    :type new_properties: Dict[str, PropertyValue]

    :raises:
        * :class:`~psym.exceptions.EntityNotFoundError`: :attr:`~psym.common.data_class.EquipmentPortDefinition.port_type_name` is None,
          there are no properties or there any unknown property name in `new_properties` keys
        * FailedOperationException: Internal symphony error

    :return: EquipmentPort object
    :rtype: :class:`~psym.common.data_class.EquipmentPort`

    **Example**

    .. code-block:: python

        location = client.get_location(location_hirerchy=[("Country", "LS_IND_Prod_Copy")])
        equipment = client.get_equipment(name="indProdCpy1_AIO", location=location)
        edited_port = client.edit_port_properties(
            equipment=equipment,
            port_name="Z AIO - Port 1",
            new_properties={"Port Property 2": "test_it"},
        )
    """
    port = get_port(client, equipment, port_name)
    port_definition_id = port.definition.id
    if not port_definition_id:
        raise EntityNotFoundError(
            entity=Entity.EquipmentPort, msg="port definition has no ID"
        )
    new_property_inputs = []
    if new_properties:
        port_type_name = port.definition.port_type_name
        if port_type_name is None:
            raise EntityNotFoundError(
                entity=Entity.Property,
                msg=f"Not possible to edit properties in '{port.definition.name}' port with undefined PortType",
            )
        property_types = PORT_TYPES[port_type_name].property_types
        new_property_inputs = get_graphql_property_inputs(
            property_types, new_properties
        )

    result = EditEquipmentPortMutation.execute(
        client,
        EditEquipmentPortInput(
            side=LinkSide(equipment=equipment.id, port=port_definition_id),
            properties=new_property_inputs,
        ),
    )
    port_type = result.definition.portType
    link = result.link
    visible_label = result.definition.visibleLabel
    _index = result.definition.index
    return EquipmentPort(
        id=result.id,
        properties=result.properties,
        definition=EquipmentPortDefinition(
            id=result.definition.id,
            name=result.definition.name,
            port_type_name=port_type.name if port_type else None,
            visible_label=visible_label if visible_label else None,
            port_definition_index=_index if _index else None,
            connected_ports=[
                EquipmentPortDefinitionAlias(id=cp.id, name=cp.name)
                for cp in result.definition.connectedPorts
            ],
        ),
        link=Link(
            id=link.id,
            properties=link.properties,
            service_ids=[s.id for s in link.services],
        )
        if link
        else None,
    )


def edit_link_properties(
    client: SymphonyClient,
    equipment: Equipment,
    port_name: str,
    new_link_properties: Dict[str, PropertyValue],
) -> EquipmentPort:
    """This function returns edited port in equipment based on its name.

    :param equipment: List of property definitions
    :type equipment: :class:`~psym.common.data_class.Equipment`
    :param port_name: Equipment type name
    :type port_name: str
    :param new_link_properties: Dictionary of property name to property value

        * str - property name
        * PropertyValue - new value of the same type for this property

    :type new_link_properties: Dict[str, PropertyValue], optional

    :raises:
        * :class:`~psym.exceptions.EntityNotFoundError`: :attr:`~psym.common.data_class.EquipmentPortDefinition.port_type_name` is None,
          there are no properties or there any unknown property name in `new_link_properties` keys
        * FailedOperationException: Internal symphony error

    :return: EquipmentPort object
    :rtype: :class:`~psym.common.data_class.EquipmentPort`

    **Example**

    .. code-block:: python

        location = client.get_location(location_hirerchy=[("Country", "LS_IND_Prod_Copy")])
        equipment = client.get_equipment(name="indProdCpy1_AIO", location=location)
        edited_port = client.edit_link_properties(
            equipment=equipment,
            port_name="Z AIO - Port 1",
            new_link_properties={"Link Property 1": 98765},
        )
    """
    port = get_port(client, equipment, port_name)
    link = port.link
    if link is None:
        raise EntityNotFoundError(entity=Entity.Link, entity_name=port_name)

    definition_port_type_name = ""
    if port.definition.port_type_name is None:
        raise EntityNotFoundError(
            entity=Entity.Property,
            msg=f"Not possible to edit link properties in '{port.definition.name}' port with undefined PortType",
        )
    else:
        definition_port_type_name = port.definition.port_type_name
    new_link_property_inputs = []
    if new_link_properties and definition_port_type_name:
        link_property_types = PORT_TYPES[definition_port_type_name].link_property_types
        new_link_property_inputs = get_graphql_property_inputs(
            link_property_types, new_link_properties
        )

    result = EditLinkMutation.execute(
        client,
        EditLinkInput(
            id=link.id, properties=new_link_property_inputs, serviceIds=link.service_ids
        ),
    )
    return EquipmentPort(
        id=port.id,
        properties=port.properties,
        definition=EquipmentPortDefinition(
            id=port.definition.id,
            name=port.definition.name,
            port_type_name=port.definition.port_type_name,
            visible_label=port.definition.visible_label,
            port_definition_index=port.definition.port_definition_index,
            connected_ports=port.definition.connected_ports,
        ),
        link=Link(
            id=result.id,
            properties=result.properties,
            service_ids=[s.id for s in result.services],
        )
        if result
        else None,
    )



#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.


from psym.client import SymphonyClient
from psym.common.data_class import CounterFamily

from ..graphql.input.edit_counter_family_input import EditCounterFamilyInput
from ..graphql.input.add_counter_family_input import AddCounterFamilyInput
from ..graphql.mutation.add_counter_family import addCounterFamily
from ..graphql.mutation.edit_counter_family import editCounterFamily
from ..graphql.mutation.remove_counter_family import removeCounterFamily
from ..graphql.query.counter_families import counterFamilies
from psym.common.constant import PAGINATION_STEP
from typing import Any, Dict, Iterator, List, Optional




def add_counter_family(
    client: SymphonyClient, name: str
) -> CounterFamily:
    """This function adds Counter Family.

    :param name: Counter Family name
    :type name: str

    :return: CounterFamily object
    :rtype: :class:`~psym.common.data_class.counterFamily`

    **Example 1**

    .. code-block:: python

        new_counter_family = client.add_counter_family(
            name="counter_family",
        )
        print(new_counter_family)
    """
    counter_family_input = AddCounterFamilyInput(name=name)
    if (name == ""):
      print ("you cant leave the name parameter empty")
    elif name!=str:
      print ("the input must be a text, example name=''new name''")
    else:
        result = addCounterFamily.execute(client, input=counter_family_input)
        return CounterFamily(name=result.name, id=result.id)


def edit_counter_family(
    client: SymphonyClient,
    counter_family: CounterFamily,
    new_name: Optional[str] = None,
) -> None:
    """This function edits Counter Family.

    :param counter_family: Counter Family entity
    :type name: str
    :param new_name: Counter Family name
    :type name: str

    :return: none object
    :rtype: :class:`~psym.common.data_class.counterFamily`

    **Example 1**

    .. code-block:: python

        counter_family_edited = client.edit_counter_family(
            counter_family=new_counter_family,
            new_name="counter_family_edited",
        )
        print(counter_family_edited)
    """
    params: Dict[str, Any] = {}
    if new_name is not None:
        params.update({"_name_": new_name})
    if new_name is not None:
        editCounterFamily.execute(client, input=EditCounterFamilyInput(id=counter_family.id, name=new_name))


def get_counter_families(client: SymphonyClient) -> Iterator[CounterFamily]:
    """ this funtion Get Counter Families


    :return: alarmStatus object
    :rtype: Iterator[ :class:`~psym.common.data_class.counterFamily` ]

    **Example**

    .. code-block:: python

        get_counter_familiies = client.get_counter_families()
        for get_counter_family in get_counter_familiies:
            print(get_counter_familiy.name)
    """
    counter_familiess_ = counterFamilies.execute(client, first=PAGINATION_STEP)
    edges = counter_familiess_.edges if counter_familiess_ else []
    while counter_familiess_ is not None and counter_familiess_.pageInfo.hasNextPage:
        counter_familiess_ = counterFamilies.execute(
            client, after=counter_familiess_.pageInfo.endCursor, first=PAGINATION_STEP
        )
        if counter_familiess_ is not None:
            edges.extend(counter_familiess_.edges)

    for edge in edges:
        node = edge.node
        if node is not None:
            yield CounterFamily(
                id=node.id,
                name=node.name,
            )


def remove_counter_family(client: SymphonyClient, id: str) -> None:

    """This function delete Counter Family.

    :param name: Counter Family name
    :type name: :class:`~psym.common.data_class.counterFamily`
    :rtype: None

    **Example**

    .. code-block:: python

        remove_counterFamilyclient.remove_counter_family(id=123456789)
    """
    removeCounterFamily.execute(client, id=id)
    


#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from functools import partial
from ...config import custom_scalars, datetime
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional

from dataclasses_json import DataClassJsonMixin, config

from gql_client.runtime.enum_utils import enum_field_metadata
from ..enum.variable_type import VariableType


@dataclass(frozen=True)
class VariableDefinitionInput(DataClassJsonMixin):
    key: str
    type: VariableType = _field(metadata=enum_field_metadata(VariableType))
    choices: List[str]
    mandatory: Optional[bool] = None
    multipleValues: Optional[bool] = None
    defaultValue: Optional[str] = None



#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.


from psym.client import SymphonyClient
from psym.common.data_class import Vendor

from ..graphql.input.edit_vendor_input import EditVendorInput
from ..graphql.input.add_vendor_input import AddVendorInput
from ..graphql.mutation.add_vendor import addVendor
from ..graphql.mutation.edit_vendor import editVendor
from ..graphql.mutation.remove_vendor import removeVendor
from ..graphql.query.vendors import vendors
from psym.common.constant import PAGINATION_STEP
from typing import Any, Dict, Iterator, List, Optional




def add_vendor(
    client: SymphonyClient, name: str
) -> Vendor:
    """This function adds vendor.

    :param name: vendor name
    :type name: str

    :return: vendor object
    :rtype: :class:`~psym.common.data_class.vendor`


    **Example 1**

    .. code-block:: python

        new_vendor = client.add_vendor(
            name="vendor",
        )
        print(new_vendor)
    """
    vendor_input = AddVendorInput(name=name)
    if (len(name) != 0):
        result = addVendor.execute(client, input=vendor_input)
        return Vendor(name=result.name, id=result.id)
    else:
        print ("you cant leave the name parameter empty")

def edit_vendor(
    client: SymphonyClient,
    vendor: Vendor,
    new_name: Optional[str] = None,
) -> None:
    """This function edits vendor.

    :param vendor: vendor entity
    :type name: str
    :param new_name: vendor name
    :type name: str

    :return: none object
    :rtype: :class:`~psym.common.data_class.vendor`

    **Example 1**

    .. code-block:: python

        vendor_edited = client.edit_vendor(
            vendor=vendor,
            new_name="new_vendor")
    """
    params: Dict[str, Any] = {}
    if new_name is not None:
        params.update({"_name_": new_name})
    if new_name is not None:
        editVendor.execute(client, input=EditVendorInput(id=vendor.id, name=new_name))


def get_vendors(client: SymphonyClient) -> Iterator[Vendor]:
    """ this funtion Get vendors


    :return: vendor object
    :rtype: Iterator[ :class:`~psym.common.data_class.vendor` ]

    **Example**

    .. code-block:: python

        vendors = client.get_vendors()
        for vendor in vendors:
            print(vendor.name)
    """
    vendors_ = vendors.execute(client, first=PAGINATION_STEP)
    edges = vendors_.edges if vendors_ else []
    while vendors_ is not None and vendors_.pageInfo.hasNextPage:
        vendors_ = vendors.execute(
            client, after=vendors_.pageInfo.endCursor, first=PAGINATION_STEP
        )
        if vendors_ is not None:
            edges.extend(vendors_.edges)

    for edge in edges:
        node = edge.node
        if node is not None:
            yield Vendor(
                id=node.id,
                name=node.name,
            )


def remove_vendor(client: SymphonyClient, id: str) -> None:
    """This function delete vendor.

    :param name: vendor name
    :type name: :class:`~psym.common.data_class.vendor`
    :rtype: None

    **Example**

    .. code-block:: python

        client.remove_vendor(id=123456789)
    """
    removeVendor.execute(client, id=id)


#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from enum import Enum


class CellularNetworkType(Enum):
    CDMA = "CDMA"
    GSM = "GSM"
    LTE = "LTE"
    WCDMA = "WCDMA"
    MISSING_ENUM = ""

    @classmethod
    def _missing_(cls, value: object) -> "CellularNetworkType":
        return cls.MISSING_ENUM



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from ...config import custom_scalars, datetime
from gql_client.runtime.variables import encode_variables
from gql import gql, Client
from gql.transport.exceptions import TransportQueryError
from functools import partial
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional
from time import perf_counter
from dataclasses_json import DataClassJsonMixin, config

from ..input.add_formula_input import AddFormulaInput


# fmt: off
QUERY: List[str] = ["""
mutation addFormula($input: AddFormulaInput!) {
  addFormula(input: $input) {
    id
    textFormula
    status
    techFk{id}
    networkTypeFk{id}
    kpiFk{id}    
  }
}
"""
]


class addFormula:
    @dataclass(frozen=True)
    class addFormulaData(DataClassJsonMixin):
        @dataclass(frozen=True)
        class Formula(DataClassJsonMixin):
            @dataclass(frozen=True)
            class Tech(DataClassJsonMixin):
                id: str

            @dataclass(frozen=True)
            class NetworkType(DataClassJsonMixin):
                id: str

            @dataclass(frozen=True)
            class Kpi(DataClassJsonMixin):
                id: str

            id: str
            textFormula: str
            status: bool
            techFk: Tech
            networkTypeFk: NetworkType
            kpiFk: Kpi

        addFormula: Formula

    # fmt: off
    @classmethod
    def execute(cls, client: Client, input: AddFormulaInput) -> addFormulaData.Formula:
        variables: Dict[str, Any] = {"input": input}
        new_variables = encode_variables(variables, custom_scalars)
        response_text = client.execute(
            gql("".join(set(QUERY))), variable_values=new_variables
        )
        res = cls.addFormulaData.from_dict(response_text)
        return res.addFormula

    # fmt: off
    @classmethod
    async def execute_async(cls, client: Client, input: AddFormulaInput) -> addFormulaData.Formula:
        variables: Dict[str, Any] = {"input": input}
        new_variables = encode_variables(variables, custom_scalars)
        response_text = await client.execute_async(
            gql("".join(set(QUERY))), variable_values=new_variables
        )
        res = cls.addFormulaData.from_dict(response_text)
        return res.addFormula
